//Joshua Stough
//November 20, 1999
//CS 317, goldfeather

#include <GL/glut.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <stdlib.h>

const int S_SIZE = 120, T_SIZE = 12;

struct clip {
        int xleft, xright, ybottom, ytop;
	double box_xl, box_xr, box_yb, box_yt, box_zh, box_zy, O, A;
};

class transformation {
   public:
        transformation ();
        ~transformation();
        void Compose(transformation *M); //Home->Compose(M) = M.Home.
	void iCompose(transformation *M); //A->iCompose(B) = A.B.
        void Translation (double scale_x, double scale_y, double scale_z,
                        int trans_x, int trans_y, int trans_z);
        void xRotation (double sine, double cosine);
        void yRotation (double sine, double cosine);
        void zRotation (double sine, double cosine);
        void show_matrix();
        void perspective_utility(clip &w);
        void Apply (double invertexlist[], int verticies);
		void Get_Mat(GLdouble mat[]);
 
  private:
        double m[4][4];
        void deg_to_rad(double &angle);
};

struct vertex {
	double x1, x2, x3, _s, _t, norm[3];
};

class edge {
   public:
       	int v1s, v2s, v1t, v2t;
	edge *next;
 
        edge (const int &x, const int &y, const int &z, const int &w);
};

class surface {
   public:
	surface ();
	~surface();
	void kill_surf();
	void draw_surf(int tri_or_wire);
			//draws the surface as seen by the eye.
			//Modified: Now just calls glVertex3d repeatedly.
	void draw_t_plane(int tri_or_wire);
			//draws the tangent plane and normal at s, t 
			//as seen by the eye.
			//Modified: As draw_surf.
	void _TP(int &i, int &j);
			//the function called from the main program to
			//update the displayed t-plane to a diff. s,t val.
	void extremes (double &x, double &y, double &u, double &v);
	void eye_extremes_util(transformation *Current);
	void print_surf(int choice);
	void write_surf(const char *filename);
	void give_st(double &s, double &t);
	int draw_Arrays_count();

   private:
	double x[10], y[10], z[10]; 	//The polynomial representation.
	int deg, s_min, s_max, t_min, t_max, tpkey_i, tpkey_j;	//the s and t ranges.
	double min_x, min_y, max_x, max_y;

	edge *surf_front, *plane_front;		//the edge lists for the surface
	
						//and the tangent plane.
	vertex SU[S_SIZE][S_SIZE], TP[T_SIZE+1][T_SIZE+1], tp_norm1, tp_norm2;
						//the surface and tangent plane
						//data.
	double gl_Vert_Array[3*(2*S_SIZE*S_SIZE - 3*S_SIZE + 1)];
	double gl_Norm_Array[3*(2*S_SIZE*S_SIZE - 3*S_SIZE + 1)];
	bool gl_Edge_Array[2*S_SIZE*S_SIZE - 4];
						//Hopefully, for faster rendering.
	double R (double var[], double &s, double &t);
					//To calculate the data at s, t.
	void copy_vert_s (int &s, int &t, double temp[]);
	void copy_vert_t (int &s, int &t, double temp[]);
	void test_for_extremes(double &x, double &y, double &u, double &v);
	void get_xyz(); 
	void recreate_t_plane();
			//To create the tangent plane, for a diff. point on the surf.
	void Gram_Schmidt(double rs[], double rt[], int &bad);
			 //For making the basis of the tangent plane orthonormal.
	void deg0_undef(double var[]);
			//So that the user can specify a divide by zero, but it doesn't 
			//affect anything.
	void find_normal(int i, int j, double n[]);
			//returns in n the vector normal to the surface at the vertex
			//given in the i-j th entry of SU. 

};	

transformation::transformation() 
{
   int i, j;
   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
                m[i][j] = 0;
        }
   }

}

transformation::~transformation()
{
}

void transformation::Compose(transformation *M)
{
   
int i, j, k;
   double sum = 0.0;
   transformation temp;

   for (i = 0; i < 4; i ++) {
      for (j = 0; j < 4; j ++) {
         for (k = 0; k < 4; k ++) {
                sum += M->m[i][k] * m[k][j];
         }
         temp.m[i][j] = sum;
         sum = 0;
      }
   }

   for (i = 0; i < 4; i ++) {
      for (j = 0; j < 4; j ++) {
        m[i][j] = temp.m[i][j];
      }
   }

}

void transformation::iCompose(transformation *M)
{

int i, j, k;
   double sum = 0;
   transformation temp;

   for (i = 0; i < 4; i ++) {
      for (j = 0; j < 4; j ++) {
         for (k = 0; k < 4; k ++) {
                sum += m[i][k] * M->m[k][j];
         }
         temp.m[i][j] = sum;
         sum = 0;
      }
   }

   for (i = 0; i < 4; i ++) {
      for (j = 0; j < 4; j ++) {
        m[i][j] = temp.m[i][j];
      }
   }

}

void transformation::Translation (double scale_x, double scale_y, 
                double scale_z, int trans_x, int trans_y, int trans_z)
//this creates and returns a translation transformation with the 
//properties of the parameters.
{
   int i, j;

   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
                if (i == 0) {
                   if (j == 0) m[i][j] = scale_x; 
                   else if (j == 3) m[i][j] = trans_x;
                   else m[i][j] = 0;
                }else if (i == 1) {
                   if (j == 1) m[i][j] = scale_y;
                   else if (j == 3) m[i][j] = trans_y;
                   else m[i][j] = 0;
                }else if (i == 2) {
                   if (j == 2) m[i][j] = scale_z;
                   else if (j == 3) m[i][j] = trans_z;
                   else m[i][j] = 0;
                }else { 
                   if (j == 3) m[i][j] = 1;
                   else m[i][j] = 0;
                }
        }
   }

}

void transformation::xRotation (double sine, double cosine)
//This assigns the elements of the matrix to represent a transformation 
//about the chosen axis, by the angle
{
   int i, j;

   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
                if (i == 0) {
                   if (j == 0) m[i][j] = 1;
                   else m[i][j] = 0;
                }else if (i == 1) {
                   if (j == 1) m[i][j] = cosine;
                   else if (j == 2) m[i][j] = sine;
                   else m[i][j] = 0;
                }else if (i == 2) {
                   if (j == 1) m[i][j] = - sine;
                   else if (j == 2) m[i][j] = cosine;
                   else m[i][j] = 0;
                }else {
                   if (j == 3) m[i][j] = 1;
                   else m[i][j] = 0;
                }
        }
   }
}

void transformation::yRotation (double sine, double cosine)
//Same for a rotation about the y axis.
{
   int i, j;

   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
                if (i == 0) {
                   if (j == 0) m[i][j] = cosine;
                   else if (j == 2) m[i][j] = - sine;
                   else m[i][j] = 0;
                }else if (i == 1) {
                   if (j == 1) m[i][j] = 1;
                   else m[i][j] = 0;
                }else if (i == 2) {
                   if (j == 0) m[i][j] = sine;
                   else if (j == 2) m[i][j] = cosine;
                   else m[i][j] = 0;
                }else {
                   if (j == 3) m[i][j] = 1;
                   else m[i][j] = 0;
                }
        }
   }
}

void transformation::zRotation (double sine, double cosine)
//Same for a rotation about the z axis.
{ 
   int i, j;

   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
                if (i == 0) {
                   if (j == 0) m[i][j] = cosine;
                   else if (j == 1) m[i][j] = - sine;
                   else m[i][j] = 0;       
                }else if (i == 1) {
                   if (j == 0) m[i][j] = sine;
                   else if (j == 1) m[i][j] = cosine;
                   else m[i][j] = 0;
                }else if (i == 2) {
                   if (j == 2) m[i][j] = 1;     
                   else m[i][j] = 0;
                }else { 
                   if (j == 3) m[i][j] = 1;     
                   else m[i][j] = 0;    
                }       
        }       
   }
}

void transformation::Apply (double invertexlist[], int verticies)
{
   int coords = 4*verticies, i, j;
   double temp[4];

   for (i = 0; i < coords; i += 4) {
        for (j = 0; j < 4; j ++) {
           temp[j] = invertexlist[i + j];
        }
        
        for (j = 0; j < 4; j ++) { 
           invertexlist[i + j] = m[j][0]*temp[0] + m[j][1]*temp[1] + 
                                m[j][2]*temp[2] + m[j][3]*temp[3];
        }
   }

}

void transformation::deg_to_rad(double &angle) 
{
   double temp = angle, pi = 3.141596;
   
   angle = (double) pi*temp/180.0;
}

void transformation::show_matrix()
//prints the matrix of transformation.
{
   int i, j;
   //cerr << "\n\n about to show a matrix\n\n";
   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
                if (j == 3) cerr << m[i][j] << "\n";
                else cerr << m[i][j] << "    ";
        }
   }

}

void transformation::perspective_utility(clip &w)
//This is a utility for the perspective matrix.  It simply puts makes the 
//fourth row consistent with saving the z value in its element.
{
   double a = (double) 1000.0*w.box_zy/(w.box_zy - w.box_zh),
        b = -a*w.box_zh;

   m[3][2] = m[2][2];
   m[3][3] = 0;
   m[2][2] = a;
   m[2][3] = b;

//And a calculation of A, the screen Y.

   w.A = (double) (a*w.box_zy + b)/w.box_zy; 
     //cerr << "A is " << w.A << " or : " << a*w.box_zy << " plus " << b 
     //  << " and the sum divided by " << w.box_zy << endl; 
}

void transformation::Get_Mat(GLdouble mat[])
{
   int i, j;
   //cerr << "\n\n about to show a matrix\n\n";
   for (i = 0; i < 4; i ++) {
        for (j = 0; j < 4; j ++) {
				mat[4*i+j] = m[j][i];
				//In OpenGL Matricies, the i-j element is ith column, jth row.
                //if (j == 3) cerr << m[i][j] << "\n";
                //else cerr << m[i][j] << "    ";
        }
   }

}

edge::edge (const int &x, const int &y, const int &z, const int &w)
{
	v1s = x;
	v1t = y;
	v2s = z;
	v2t = w;
	next = NULL;
}

surface::~surface()
{
	kill_surf();
}

void surface::kill_surf()
{
	edge *kill;
	
	while (surf_front != NULL)
	{
		kill = surf_front;
		surf_front = surf_front->next;
		delete kill;
	}

	while (plane_front != NULL)
	{
		kill = plane_front;
		plane_front = plane_front->next;
		delete kill;
	}	
}
	
double surface::R (double var[], double &s, double &t)
//Calculating x,y, or z at (s,t).
{
   double sum; 

   if (deg == 2) {
	sum = (double) var[0]*pow(s,2) + var[1]*s*t + var[2]*pow(t,2) 
		+ var[3]*s + var[4]*t + var[5];
   }else if (deg == 3) {
	sum = (double) var[0]*pow(s,3) + var[1]*pow(s,2)*t + var[2]*s*pow(t,2)
		+ var[3]*pow(t,3) + var[4]*pow(s,2) + var[5]*s*t + var[6]*pow(t,2)
		+ var[7]*s + var[8]*t + var[9];
   }else if (deg == 0) {
	   sum = (double) var[0]*s + var[1]*t 
		   + var[2]*cos((pow(s,2) + pow(t,2))/var[3]) + var[4]*cos(s*t/var[5]) 
		   + var[6]*cos(s/var[7]) + var[8]*cos(t/var[9]);
   }

//cerr << "z at " << s << " and " << t << " is " << sum    
//     << "\n ...\n";

   return sum;
}

void surface::get_xyz()
//This goes through the trouble of entering the x, y, and z parametric 
//equations, to alleviate the clutter in the constructor (below)
{
   int response;

   	cout << "This app allows you to view simple quadratic, cubic,\n"
	     << "functions in s and t. Press 2 if you want to enter a \n"
		 << "quadratic, 3 for a cubic, or 0 if you'd like my random\n"
		 << "mix of linear, quadratic, and trigonometric terms.\n";
        cin >> response;

	if (response == 2) {
		deg = 2;
		cout << "f(s,t) = As^2 + Bst + Ct^2 + Ds + Et + F\n"
             	     << "X(s,t): Please enter the coefficients A B C D E F\n";
		cin >> x[0] >> x[1] >> x[2] >> x[3] >> x[4] >> x[5];
		cout << "Y(s,t): Please enter the coefficients A B C D E F\n";
		cin >> y[0] >> y[1] >>y[2] >> y[3] >> y[4] >> y[5];
		cout << "Z(s,t): Please enter the coefficients A B C D E F\n";
		cin >> z[0] >> z[1] >> z[2] >> z[3] >> z[4] >> z[5];

	}else if (response == 3) {
		deg = 3;
		cout << "f(s,t) = As^3 + Bs^2*t + Cs*t^2 + Dt^3 + Es^2 + Fst + Gt^2 + Hs + It + J\n"
		     << "X(s,t): Please enter the coefficients A B C D E F G H I J\n";
		cin >> x[0] >> x[1] >> x[2] >> x[3] >> x[4] >> x[5] >> x[6] >> x[7] >> x[8] >> x[9];
		cout << "Y(s,t): Please enter the coefficients A B C D E F G H I J\n";	
		cin >> y[0] >> y[1] >>y[2] >> y[3] >> y[4] >> y[5] >> y[6] >> y[7] >> y[8] >> y[9];
		cout << "Z(s,t): Please enter the coefficients A B C D E F G H I J\n";
		cin >> z[0] >> z[1] >> z[2] >> z[3] >> z[4] >> z[5] >> z[6] >> z[7] >> z[8] >> z[9];
	}else if (response == 0) {
		deg = 0;
		cout << "f(s,t) = As + Bt + Ccos((s^2 + t^2)/D) + Ecos(st/F) + Gcos(s/H) + Icos(t/J)\n"
			 << "X(s,t): Please enter the coefficients A B C D E F G H I J\n";
		cin >> x[0] >> x[1] >> x[2] >> x[3] >> x[4] >> x[5] >> x[6] >> x[7] >> x[8] >> x[9];
		deg0_undef(x);
		cout << "Y(s,t): Please enter the coefficients A B C D E F G H I J\n";	
		cin >> y[0] >> y[1] >>y[2] >> y[3] >> y[4] >> y[5] >> y[6] >> y[7] >> y[8] >> y[9];
		deg0_undef(y);
		cout << "Z(s,t): Please enter the coefficients A B C D E F G H I J\n";
		cin >> z[0] >> z[1] >> z[2] >> z[3] >> z[4] >> z[5] >> z[6] >> z[7] >> z[8] >> z[9];
		deg0_undef(z);
	}
	

	cout << "Now enter the ranges for s and t\n"
             << "s_min s_max t_min t_max\n";
        cin >> s_min >> s_max >> t_min >> t_max;
}

surface::surface()
{
	int i, j, LIMIT = S_SIZE -1;
	double s, t, s_inc, t_inc;
   	edge *curpent;   
	surf_front = NULL; plane_front = NULL;
	tpkey_i = 0; tpkey_j = 0;

	get_xyz();	

	s_inc = (double) (s_max - s_min)/S_SIZE;
	t_inc = (double) (t_max - t_min)/S_SIZE;
 	s = s_min - s_inc;
	t = t_min - t_inc;

	for (i = 0; i < S_SIZE; i ++) {
		s += s_inc;
	   for (j = 0; j < S_SIZE; j ++) {
		t += t_inc; 
	
		SU[i][j].x1 = R(x, s, t);
		SU[i][j].x2 = R(y, s, t);
		SU[i][j].x3 = R(z, s, t);
		SU[i][j]._s = s;
		SU[i][j]._t = t; 
		find_normal(i, j, SU[i][j].norm);
	   }
		t = t_min - t_inc;
		if (i%20 == 0) cout << ".";
	}
	
	surf_front = new edge(0, 0, 1, 0);
	curpent = surf_front;

	for (i = 1; i < LIMIT; i++) {
		curpent->next = new edge (i, 0, i+1, 0);
		curpent = curpent->next;
	}

	for (j = 1; j < S_SIZE; j ++) { 
	    for (i = 0; i < LIMIT; i ++) {
		curpent->next = new edge (i, j, i, j - 1);
		curpent = curpent->next;
		curpent->next = new edge (i, j, i+1, j-1);
		curpent = curpent->next;
		curpent->next = new edge (i, j, i+1, j);
		curpent = curpent->next;
	    }
	}

	for (j = 0; j < LIMIT; j ++) {
		curpent->next = new edge (LIMIT, j, LIMIT, j+1);
		curpent = curpent->next;
	}

//Now the surface edgelist exists.  Now one could create the tangent plane
//edgelist.

}

void surface::copy_vert_s (int &s, int &t, double temp[])
{
	temp[0] = SU[s][t].x1;
	temp[1] = SU[s][t].x2;
	temp[2] = SU[s][t].x3;
	temp[3] = 1;
}

void surface::copy_vert_t (int &s, int &t, double temp[])
{
	temp[0] = TP[s][t].x1;
        temp[1] = TP[s][t].x2;
        temp[2] = TP[s][t].x3;
        temp[3] = 1;
}

void surface::draw_surf(int tri_or_wire)
{
	int i, j;
	edge *curpent = surf_front;
	glColor3f(0.0,0.0,1.0);
/*
glNewList(1, GL_COMPILE);
	if (tri_or_wire < 0) {
	
		glBegin(GL_LINES);
		while (curpent != NULL) {
			glVertex3d(SU[curpent->v1s][curpent->v1t].x1, SU[curpent->v1s][curpent->v1t].x2, SU[curpent->v1s][curpent->v1t].x3);
			glVertex3d(SU[curpent->v2s][curpent->v2t].x1, SU[curpent->v2s][curpent->v2t].x2, SU[curpent->v2s][curpent->v2t].x3);
			curpent = curpent->next;
		}
		glEnd();
		glFlush();
	}else if (tri_or_wire > 0) {
		glBegin(GL_TRIANGLES);

		for (i = 0; i < S_SIZE - 1; i++) {
			for (j = 0; j < S_SIZE - 1; j ++) {
				glNormal3dv(&SU[i][j].norm[0]);
				glVertex3d(SU[i][j].x1, SU[i][j].x2, SU[i][j].x3);

				glNormal3dv(&SU[i][j+1].norm[0]);
				glVertex3d(SU[i][j+1].x1, SU[i][j+1].x2, SU[i][j+1].x3);

				glNormal3dv(&SU[i+1][j].norm[0]);
				glVertex3d(SU[i+1][j].x1, SU[i+1][j].x2, SU[i+1][j].x3);

				glNormal3dv(&SU[i+1][j].norm[0]);
				glVertex3d(SU[i+1][j].x1, SU[i+1][j].x2, SU[i+1][j].x3);

				glNormal3dv(&SU[i+1][j+1].norm[0]);
				glVertex3d(SU[i+1][j+1].x1, SU[i+1][j+1].x2, SU[i+1][j+1].x3);

				glNormal3dv(&SU[i][j+1].norm[0]);
				glVertex3d(SU[i][j+1].x1, SU[i][j+1].x2, SU[i][j+1].x3);
			}
		}
		glEnd();
		glFlush();
	}
glEndList();
*/
	
	int q = 0;
	
			gl_Edge_Array[q] = true;
			gl_Vert_Array[3*q] = SU[0][0].x1;
			gl_Norm_Array[3*q] = SU[0][0].norm[0];
			gl_Vert_Array[3*q + 1] = SU[0][0].x2;
			gl_Norm_Array[3*q + 1] = SU[0][0].norm[1];
			gl_Vert_Array[3*q + 2] = SU[0][0].x3;
			gl_Norm_Array[3*q + 2] = SU[0][0].norm[2];

			q++;
			gl_Edge_Array[q] = true;
			gl_Vert_Array[3*q] = SU[1][0].x1;
			gl_Norm_Array[3*q] = SU[1][0].norm[0];
			gl_Vert_Array[3*q + 1] = SU[1][0].x2;
			gl_Norm_Array[3*q + 1] = SU[1][0].norm[1];
			gl_Vert_Array[3*q + 2] = SU[1][0].x3;
			gl_Norm_Array[3*q + 2] = SU[1][0].norm[2];
			
			q++;

	for(i = 0; i <= S_SIZE-2; i = i + 2) {
		for (j = 1; j <= S_SIZE-1; j ++) {
			gl_Edge_Array[q] = true;
			gl_Vert_Array[3*q] = SU[i][j].x1;
			gl_Norm_Array[3*q] = SU[i][j].norm[0];
			gl_Vert_Array[3*q + 1] = SU[i][j].x2;
			gl_Norm_Array[3*q + 1] = SU[i][j].norm[1];
			gl_Vert_Array[3*q + 2] = SU[i][j].x3;
			gl_Norm_Array[3*q + 2] = SU[i][j].norm[2];

			q++;
			gl_Edge_Array[q] = true;
			gl_Vert_Array[3*q] = SU[i+1][j].x1;
			gl_Norm_Array[3*q] = SU[i+1][j].norm[0];
			gl_Vert_Array[3*q + 1] = SU[i+1][j].x2;
			gl_Norm_Array[3*q + 1] = SU[i+1][j].norm[1];
			gl_Vert_Array[3*q + 2] = SU[i+1][j].x3;
			gl_Norm_Array[3*q + 2] = SU[i+1][j].norm[2];
			
			q++;
		}

			gl_Edge_Array[q] = false;
			gl_Vert_Array[3*q] = SU[i+2][S_SIZE-1].x1;
			gl_Norm_Array[3*q] = SU[i+2][S_SIZE-1].norm[0];
			gl_Vert_Array[3*q + 1] = SU[i+2][S_SIZE-1].x2;
			gl_Norm_Array[3*q + 1] = SU[i+2][S_SIZE-1].norm[1];
			gl_Vert_Array[3*q + 2] = SU[i+2][S_SIZE-1].x3;
			gl_Norm_Array[3*q + 2] = SU[i+2][S_SIZE-1].norm[2];

			q++;


		for(j = S_SIZE-2; j >= 0; j--) {
			gl_Edge_Array[q] = true;
			gl_Vert_Array[3*q] = SU[i+1][j].x1;
			gl_Norm_Array[3*q] = SU[i+1][j].norm[0];
			gl_Vert_Array[3*q + 1] = SU[i+1][j].x2;
			gl_Norm_Array[3*q + 1] = SU[i+1][j].norm[1];
			gl_Vert_Array[3*q + 2] = SU[i+1][j].x3;
			gl_Norm_Array[3*q + 2] = SU[i+1][j].norm[2];

			q++;
			gl_Edge_Array[q] = true;
			gl_Vert_Array[3*q] = SU[i+2][j].x1;
			gl_Norm_Array[3*q] = SU[i+2][j].norm[0];
			gl_Vert_Array[3*q + 1] = SU[i+2][j].x2;
			gl_Norm_Array[3*q + 1] = SU[i+2][j].norm[1];
			gl_Vert_Array[3*q + 2] = SU[i+2][j].x3;
			gl_Norm_Array[3*q + 2] = SU[i+2][j].norm[2];
			
			q++;
		}
			
			gl_Edge_Array[q] = false;
			gl_Vert_Array[3*q] = SU[i+3][0].x1;
			gl_Norm_Array[3*q] = SU[i+3][0].norm[0];
			gl_Vert_Array[3*q + 1] = SU[i+3][0].x2;
			gl_Norm_Array[3*q + 1] = SU[i+3][0].norm[1];
			gl_Vert_Array[3*q + 2] = SU[i+3][0].x3;
			gl_Norm_Array[3*q + 2] = SU[i+3][0].norm[2];
			
			q++;

	}

	glVertexPointer(3, GL_DOUBLE, 0, &gl_Vert_Array[0]);
	glNormalPointer(GL_DOUBLE, 0, &gl_Norm_Array[0]);
	glEdgeFlagPointer(0, &gl_Edge_Array[0]);

	glEnableClientState(GL_EDGE_FLAG_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glEnableClientState(GL_VERTEX_ARRAY);


}

void surface::draw_t_plane(int tri_or_wire)
{
	edge *curpent = plane_front;
glNewList(2,GL_COMPILE);
	glColor3f(1.0,0.0,0.0);
	glBegin(GL_LINES);
	while (curpent != NULL) {
		glVertex3d(TP[curpent->v1s][curpent->v1t].x1, TP[curpent->v1s][curpent->v1t].x2, TP[curpent->v1s][curpent->v1t].x3);
		glVertex3d(TP[curpent->v2s][curpent->v2t].x1, TP[curpent->v2s][curpent->v2t].x2, TP[curpent->v2s][curpent->v2t].x3);
		curpent = curpent->next;
	}
	glColor3f(0.0, 1.0, 0.0);
	glVertex3d(tp_norm1.x1, tp_norm1.x2, tp_norm1.x3);
	glVertex3d(tp_norm2.x1, tp_norm2.x2, tp_norm2.x3);
	glEnd();
	glFlush();
glEndList();
}

/*	
void surface::draw_surf(transformation *M, clip w)
{
        int i;
        double c[7], q[7], x0, y0, x1, y1, s[4], p[4],
                t, t_in, t_out;
        bool reject, skip;
        edge *curpent = surf_front;

        glBegin(GL_LINES);
        
	while (curpent != NULL) {
                copy_vert_s(curpent->v1s, curpent->v1t, s);
                copy_vert_s(curpent->v2s, curpent->v2t, p);
                M->Apply(s, 1);
                M->Apply(p, 1);

                c[1] = (double) p[0]/p[3] - s[0]/s[3];
                c[2] = (double) s[0]/s[3] - p[0]/p[3];
                c[3] = (double) p[1]/p[3] - s[1]/s[3];
                c[4] = (double) s[1]/s[3] - p[1]/p[3];
                c[5] = (double) p[2]/p[3] - s[2]/s[3];
                c[6] = (double) s[2]/s[3] - p[2]/p[3];
                q[1] = (double) w.box_xl - s[0]/s[3];
                q[2] = (double) s[0]/s[3] - w.box_xr;
                q[3] = (double) w.box_yb - s[1]/s[3];
                q[4] = (double) s[1]/s[3] - w.box_yt;
                q[5] = (double) -s[2]/s[3];
                q[6] = (double) s[2]/s[3] - w.A;
                reject = false;
                skip = false;
                i = 1;
                t_in = -10000;
                t_out = 10000;

                do {
                        if (c[i] == 0){
                          if (q[i] > 0) {reject = true; cout << "reject cond 1\n";}
                          else skip = true;
                        }

                        if ((skip == false) && (reject == false)) {
                          t = (double) q[i]/c[i];

                          if ((c[i] < 0) && (t_out > t)) t_out = t;
                          if ((c[i] > 0) && (t_in < t)) t_in = t;
                          if ((t_out < 0) || (t_in > 1) || (t_in > t_out))
                                {reject = true; cout << "reject cond 2\n ";}
                        }
                        i++;
                        skip = false;
                }while ((i < 7) && (reject == false));

		if (reject == false) {
                        if (t_in < 0) t_in = 0;
                        if (t_out >1) t_out = 1;
                        x0 = (double) s[0]/s[3] + t_in*c[1];
                        y0 = (double) s[1]/s[3] + t_in*c[3];
                        x1 = (double) s[0]/s[3] + t_out*c[1];
                        y1 = (double) s[1]/s[3] + t_out*c[3];

                        glVertex2i((int) x0, (int) y0);
                        glVertex2i((int) x1, (int) y1);
                }

                curpent = curpent->next;

        }
        glEnd();

}//end of the clip_and_draw function.


void surface::draw_t_plane(transformation *M, clip w)
{
        int i;
        double c[7], q[7], x0, y0, x1, y1, s[4], p[4],
                t, t_in, t_out;
        bool reject, skip;
        edge *curpent = plane_front;

	glColor3f(1.0, 0.0, 0.0);
        glBegin(GL_LINES);
        
        while (curpent != NULL) {
                copy_vert_t(curpent->v1s, curpent->v1t, s);
                copy_vert_t(curpent->v2s, curpent->v2t, p);
                M->Apply(s, 1);
                M->Apply(p, 1);

                c[1] = (double) p[0]/p[3] - s[0]/s[3];
                c[2] = (double) s[0]/s[3] - p[0]/p[3];
                c[3] = (double) p[1]/p[3] - s[1]/s[3];
                c[4] = (double) s[1]/s[3] - p[1]/p[3];
                c[5] = (double) p[2]/p[3] - s[2]/s[3];
                c[6] = (double) s[2]/s[3] - p[2]/p[3];
                q[1] = (double) w.box_xl - s[0]/s[3];
                q[2] = (double) s[0]/s[3] - w.box_xr;
                q[3] = (double) w.box_yb - s[1]/s[3];
                q[4] = (double) s[1]/s[3] - w.box_yt;
                q[5] = (double) -s[2]/s[3];
                q[6] = (double) s[2]/s[3] - w.A;
                reject = false;
                skip = false;
                i = 1;
                t_in = -10000;
                t_out = 10000;

                do {
                        if (c[i] == 0){
                          if (q[i] > 0) {reject = true; //cout << "reject cond 1\n";}
                          else skip = true;
                        }

                        if ((skip == false) && (reject == false)) {
                          t = (double) q[i]/c[i];

                          if ((c[i] < 0) && (t_out > t)) t_out = t;
                          if ((c[i] > 0) && (t_in < t)) t_in = t;
                          if ((t_out < 0) || (t_in > 1) || (t_in > t_out))
                                {reject = true; //cout << "reject cond 2\n ";}
                        }
                        i++;
                        skip = false;
                }while ((i < 7) && (reject == false));

                if (reject == false) {
                        if (t_in < 0) t_in = 0;
                        if (t_out >1) t_out = 1;
                        x0 = (double) s[0]/s[3] + t_in*c[1];
                        y0 = (double) s[1]/s[3] + t_in*c[3];
                        x1 = (double) s[0]/s[3] + t_out*c[1];
                        y1 = (double) s[1]/s[3] + t_out*c[3];

			glVertex2i((int) x0, (int) y0);
                        glVertex2i((int) x1, (int) y1);
                }

                curpent = curpent->next;

        }
        glEnd();

}//end of the clip_and_draw function.
 

void surface::draw_surf(transformation *M)
{
	edge *curpent = surf_front;
	double x, y, temp[4];

	glBegin(GL_LINES);

	while (curpent != NULL) {
//cerr << curpent->v1s << " " << curpent->v1t << " at " << i <<endl;
//if (i == 29) {
//	cout << "I'm here\n";
//}
//i++;


		copy_vert_s(curpent->v1s, curpent->v1t, temp);

		M->Apply(temp, 1);
		x = (double) temp[0]/temp[3];
                y = (double) temp[1]/temp[3];

		glVertex2d(x, y);

		copy_vert_s(curpent->v2s, curpent->v2t, temp);	
		
		M->Apply(temp, 1);
		x = (double) temp[0]/temp[3];
                y = (double) temp[1]/temp[3];

		glVertex2d(x, y);
	
		curpent = curpent->next;
	}
	glEnd();
}				
*/


void surface::_TP(int &i, int &j)
{
   if (i == 1) {
	if (tpkey_i < S_SIZE - 1) tpkey_i ++; 
   }else if (i == -1){
	if (tpkey_i > 0) tpkey_i --;
   }

   if(j == 1) {
	if (tpkey_j < S_SIZE - 1) tpkey_j ++;
   }else if (j == -1) {
	if (tpkey_j > 0) tpkey_j --;
   }
   
   recreate_t_plane();
}
/*
void surface::draw_t_plane(transformation *M)
{
   edge *curpent = plane_front;
   double x, y, temp[4];

    	glColor3f(1.0, 0, 0);
        glBegin(GL_LINES);

        while (curpent != NULL) {
                copy_vert_t(curpent->v1s, curpent->v1t, temp);

                M->Apply(temp, 1);
                x = (double) temp[0]/temp[3];
                y = (double) temp[1]/temp[3];

                glVertex2d(x, y);

                copy_vert_t(curpent->v2s, curpent->v2t, temp);

                M->Apply(temp, 1);
                x = (double) temp[0]/temp[3];
                y = (double) temp[1]/temp[3];

                glVertex2d(x, y);

                curpent = curpent->next;
        }
        glEnd();
}
*/ 
void surface::recreate_t_plane()
//This is a really BIG function! First, find the two tangent vectors
//to the surface at s and t (one with respect to s, the other to t).
//Then, orthonormalize them (so that the resulting tangent plane is 
//square.  Then, create the plane, pointing plane_front to it. 
{
   int i, j, tp_i = 0, tp_j = 0, LIMIT = T_SIZE - 1, a, b, bad;
   double rs[3], rt[3], s0 = SU[tpkey_i][tpkey_j]._s, temp1, temp2, temp3,
		t0 = SU[tpkey_i][tpkey_j]._t, x0 = SU[tpkey_i][tpkey_j].x1,
		y0 = SU[tpkey_i][tpkey_j].x2, z0 = SU[tpkey_i][tpkey_j].x3;
   edge *curpent = NULL;
//First, delete the old tangent plane.
   curpent = plane_front;
   while (curpent != NULL) {
	plane_front = plane_front->next;
	delete curpent;
  	curpent = plane_front;
   }
  

//Here is a tough part.  I've hardcoded the proper partial derivatives,
//so depending on the deg of the x y z, this is ugly but efficient.

   if (deg == 3) {
	rs[0] = (double) 3*x[0]*pow(s0,2) + 2*x[1]*s0*t0
			+ x[2]*pow(t0,2) + 2*x[4]*s0 + x[5]*t0 + x[7];
	rs[1] = (double) 3*y[0]*pow(s0,2) + 2*y[1]*s0*t0
                        + y[2]*pow(t0,2) + 2*y[4]*s0 + y[5]*t0 + y[7];
	rs[2] = (double) 3*z[0]*pow(s0,2) + 2*z[1]*s0*t0
                        + z[2]*pow(t0,2) + 2*z[4]*s0 + z[5]*t0 + z[7];
	rt[0] = (double) x[1]*pow(s0,2) + 2*x[2]*s0*t0
			+ 3*x[3]*pow(t0,2) + x[5]*s0 + 2*x[6]*t0 + x[8];
	rt[1] = (double) y[1]*pow(s0,2) + 2*y[2]*s0*t0
                        + 3*y[3]*pow(t0,2) + y[5]*s0 + 2*y[6]*t0 + y[8];
	rt[2] = (double) z[1]*pow(s0,2) + 2*z[2]*s0*t0
                        + 3*z[3]*pow(t0,2) + z[5]*s0 + 2*z[6]*t0 + z[8];
   }else if (deg == 2) {
	rs[0] = (double) 2*x[0]*s0 + x[1]*t0 + x[3];
	rs[1] = (double) 2*y[0]*s0 + y[1]*t0 + y[3];
	rs[2] = (double) 2*z[0]*s0 + z[1]*t0 + z[3];
	rt[0] = (double) x[1]*s0 + 2*x[2]*t0 + x[4];
	rt[1] = (double) y[1]*s0 + 2*y[2]*t0 + y[4];
	rt[2] = (double) z[1]*s0 + 2*z[2]*t0 + z[4];
   }else if (deg == 0) {
	temp1 = (pow(s0,2) + pow(t0,2))/x[3];
	temp1 = (-2.0*x[2]/x[3])*sin(temp1);
	temp2 = s0*t0/x[5];
	temp2 = (-1.0*x[4]/x[5])*sin(temp2);
	temp3 = s0/x[7];
	temp3 = (-1.0*x[6]/x[7])*sin(temp3);
	rs[0] = (double) x[0] + s0*temp1 + t0*temp2 + temp3;
	temp3 = t0/x[9];
	temp3 = (-1.0*x[8]/x[9])*sin(temp3);
	rt[0] = (double) x[1] + t0*temp1 + s0*temp2 + temp3;

	temp1 = (pow(s0,2) + pow(t0,2))/y[3];
	temp1 = (-2.0*y[2]/y[3])*sin(temp1);
	temp2 = s0*t0/y[5];
	temp2 = (-1.0*y[4]/y[5])*sin(temp2);
	temp3 = s0/y[7];
	temp3 = (-1.0*y[6]/y[7])*sin(temp3);
	rs[1] = (double) y[0] + s0*temp1 + t0*temp2 + temp3;
	temp3 = t0/y[9];
	temp3 = (-1.0*y[8]/y[9])*sin(temp3);
	rt[1] = (double) y[1] + t0*temp1 + s0*temp2 + temp3;

	temp1 = (pow(s0,2) + pow(t0,2))/z[3];
	temp1 = (-2.0*z[2]/z[3])*sin(temp1);
	temp2 = s0*t0/z[5];
	temp2 = (-1.0*z[4]/z[5])*sin(temp2);
	temp3 = s0/z[7];
	temp3 = (-1.0*z[6]/z[7])*sin(temp3);
	rs[2] = (double) z[0] + s0*temp1 + t0*temp2 + temp3;
	temp3 = t0/z[9];
	temp3 = (-1.0*z[8]/z[9])*sin(temp3);
	rt[2] = (double) z[1] + t0*temp1 + s0*temp2 + temp3;
   }


   Gram_Schmidt(rs, rt, bad);
 
   a = (int)  T_SIZE/2;
   b = (int)  T_SIZE/2;

   temp1 = 0.1*(fabs(s_max - s_min));
   temp1 = (double)temp1/T_SIZE;
   temp2 = 0.1*(fabs(t_max - t_min));
   temp2 = (double)temp2/T_SIZE;
   //this is so that the tangent plane isn't too big, relative to the surface.

   for (i = -a; i < a; i++) {
      	for (j = -b; j < b; j++) {

		TP[tp_i][tp_j].x1 = (double) i*rs[0]*temp1 + j*rt[0]*temp2 + x0;
		TP[tp_i][tp_j].x2 = (double) i*rs[1]*temp1 + j*rt[1]*temp2 + y0; 
		TP[tp_i][tp_j].x3 = (double) i*rs[2]*temp1 + j*rt[2]*temp2 + z0;
	
		tp_j ++;
	}
	tp_i ++;
	tp_j = 0;
   }		 


   tp_norm1.x1 = x0;tp_norm1.x2 = y0; tp_norm1.x3 = z0;
   tp_norm2.x1 = 6.0*temp1*temp2*(rs[1]*rt[2] - rt[1]*rs[2]) + x0;
   tp_norm2.x2 = -6.0*temp1*temp2*(rs[0]*rt[2] - rt[0]*rs[2]) + y0;
   tp_norm2.x3 = 6.0*temp1*temp2*(rs[0]*rt[1] - rt[0]*rs[1]) + z0; 

   plane_front = new edge(0, 0, 1, 0);
   curpent = plane_front;

   for (i = 1; i < LIMIT; i++) {
           curpent->next = new edge (i, 0, i+1, 0);
           curpent = curpent->next;
   }

   for (j = 1; j < T_SIZE; j ++) {
       for (i = 0; i < LIMIT; i ++) {
           curpent->next = new edge (i, j, i, j - 1);
           curpent = curpent->next;
           curpent->next = new edge (i, j, i+1, j-1);
           curpent = curpent->next;
           curpent->next = new edge (i, j, i+1, j);
           curpent = curpent->next;
       }
   }

   for (j = 0; j < LIMIT; j ++) {
           curpent->next = new edge (LIMIT, j, LIMIT, j+1);
           curpent = curpent->next;
   }

  
}
 
void surface::print_surf(int choice)
{ edge *current = NULL;

	if (choice == 2) {
		current = surf_front;
		while (current != NULL) {
		cout << current->v1s << " " << current->v1t << " " 
		   << current->v2s << " " << current->v2t << endl;
		current = current->next;
 		}
      	}else if (choice == 3) {
		current = plane_front;
		while (current != NULL) {
		cout << current->v1s << " " << current->v1t << " "      
                   << current->v2s << " " << current->v2t << endl;      
                current = current->next;
   		}
  	}
}

int surface::draw_Arrays_count()
{
	return (2*S_SIZE*S_SIZE - 3*S_SIZE + 1);
}

void surface::write_surf(const char *filename)
{
	ofstream fout;
	int i, j = 0, LIMIT, up_left, up_right, low_left, low_right;
	double s_lowcut = s_max + 1.0, s_highcut = s_min - 1.0;
	double t_lowcut = t_max + 1.0, t_highcut = t_min - 1.0;
	int t_width = 0, temp = 0, first_time = 0;
	char resp;
	//Think of a square of four points. Draw the two triangles using the above 
	//descriptors.  u_l, u_r, l_l, then l_l, l_r, u_r.

	cerr << "Would you like to punch a hole in the surface?(y/n)\n";
	cin >> resp;

	if (resp == 'y') {
		cerr << "Alright, provide the domain in s and t to cut.\n";
		cin >> s_lowcut >> s_highcut >> t_lowcut >> t_highcut;
		cerr << "Ready";
	}

	fout.open(filename);

	fout << "#Obj file created by Joshua Stough Graphing Calculator.\n"
		 << "#Implicitly defined surface of X, Y, and Z as functions\n"
		 << "#of s and t in the following form. (Coefficients defined.)\n";

	if (deg == 2) {
		fout << "#f(s,t) = As^2 + Bst + Ct^2 + Ds + Et + F\n";
		fout << "#X(s,t): " << x[0] << "  " << x[1] << "  " << x[2] 
			 << "  " << x[3] << "  " <<  x[4] << "  " << x[5] << endl;
		fout << "#Y(s,t): " << y[0] << "  " << y[1] << "  " << y[2] 
			 << "  " << y[3] << "  " <<  y[4] << "  " << y[5] << endl;
		fout << "#Z(s,t): " << z[0] << "  " << z[1] << "  " << z[2] 
			 << "  " << z[3] << "  " <<  z[4] << "  " << z[5] << endl;

	}else if (deg == 3) {
		fout << "#f(s,t) = As^3 + Bs^2*t + Cs*t^2 + Dt^3 + Es^2 + Fst + Gt^2 + Hs + It + J\n";
		fout << "#X(s,t): " << x[0] << "  " << x[1] << "  " << x[2] << "  " 
			 << x[3] << "  " << x[4] << "  " << x[5] << "  " << x[6] << "  "
			 << x[7] << "  " << x[8] << "  " << x[9] << endl;
		fout << "#Y(s,t): " << y[0] << "  " << y[1] << "  " << y[2] << "  " 
			 << y[3] << "  " << y[4] << "  " << y[5] << "  " << y[6] << "  "
			 << y[7] << "  " << y[8] << "  " << y[9] << endl;
		fout << "#Z(s,t): " << z[0] << "  " << z[1] << "  " << z[2] << "  " 
			 << z[3] << "  " << z[4] << "  " << z[5] << "  " << z[6] << "  "
			 << z[7] << "  " << z[8] << "  " << z[9] << endl;

	}else if (deg == 0) {
		fout<<"#f(s,t) = As + Bt + Ccos((s^2 + t^2)/D) + Ecos(st/F) + Gcos(s/H) + Icos(t/J)\n";
		fout << "#X(s,t): " << x[0] << "  " << x[1] << "  " << x[2] << "  " 
			 << x[3] << "  " << x[4] << "  " << x[5] << "  " << x[6] << "  "
			 << x[7] << "  " << x[8] << "  " << x[9] << endl;
		fout << "#Y(s,t): " << y[0] << "  " << y[1] << "  " << y[2] << "  " 
			 << y[3] << "  " << y[4] << "  " << y[5] << "  " << y[6] << "  "
			 << y[7] << "  " << y[8] << "  " << y[9] << endl;
		fout << "#Z(s,t): " << z[0] << "  " << z[1] << "  " << z[2] << "  " 
			 << z[3] << "  " << z[4] << "  " << z[5] << "  " << z[6] << "  "
			 << z[7] << "  " << z[8] << "  " << z[9] << endl;
	}

	fout << "#Domains of s and t: " << s_min << " to " << s_max 
		 << " and " << t_min << " to " << t_max << " respectively.\n";
	fout << "g Object__0\n";

	cerr<<".";
	//List the vertices and normals.

	for (i = 0; i < S_SIZE; i ++)
	{
	  if ((SU[i][j-1]._s >= s_lowcut)&&(SU[i][j-1]._s <= s_highcut)) first_time ++;
	  for (j = 0; j < S_SIZE; j ++)
	  {
		//  fout << "v " << SU[i][j].x1 << " " << SU[i][j].x2 << " " << SU[i][j].x3 << "\n";
		 // fout << "vn " << SU[i][j].norm[0] << " " << SU[i][j].norm[1] << " " 
		//	   << SU[i][j].norm[2] << "\n";
		  if ((SU[i][j]._s < s_lowcut)||(SU[i][j]._s > s_highcut)||
			  (SU[i][j]._t < t_lowcut)||(SU[i][j]._t > t_highcut))
		  {
		  fout << "v " << SU[i][j].x1 << " " << SU[i][j].x2 << " " << SU[i][j].x3 << "\n";
		  fout << "vn " << SU[i][j].norm[0] << " " << SU[i][j].norm[1] << " " 
			   << SU[i][j].norm[2] << "\n";
		  }
		  else if (first_time == 1) t_width ++;
	  }
	}
	
	fout << "usemtl (null)\n";

	cerr<<".t_width is " << t_width;
	//List the faces (triangles)

	LIMIT = S_SIZE - 2;
	for (i = 0; i < LIMIT; i ++)
	{
	  for (j = 0; j < LIMIT; j ++)
	  {
/*
		  if (((SU[i][j]._s >= s_lowcut)&&(SU[i][j]._s <= s_highcut))&&
			  ((SU[i][j]._t >= t_lowcut)&&(SU[i][j]._t <= t_highcut)))
		  {
			  up_left = 0;
			  temp ++;
		  }else up_left = i*S_SIZE + j + 1;// - temp;

		  if (((SU[i][j+1]._s >= s_lowcut)&&(SU[i][j+1]._s <= s_highcut))&&
			  ((SU[i][j+1]._t >= t_lowcut)&&(SU[i][j+1]._t <= t_highcut)))
		  {
			  up_right = 0;
		  }else up_right = i*S_SIZE + j + 1 + 1;
		  if (((SU[i+1][j]._s >= s_lowcut)&&(SU[i+1][j]._s <= s_highcut))&&
			  ((SU[i+1][j]._t >= t_lowcut)&&(SU[i+1][j]._t <= t_highcut)))
		  {
			  low_left = 0;
		  }else low_left = i*S_SIZE + j + 1 + S_SIZE;
		  if (((SU[i+1][j+1]._s >= s_lowcut)&&(SU[i+1][j+1]._s <= s_highcut))&&
			  ((SU[i+1][j+1]._t >= t_lowcut)&&(SU[i+1][j+1]._t <= t_highcut)))
		  {
			  low_right = 0;
		  }else low_right = i*S_SIZE + j + 1 + S_SIZE + 1;

		  if (up_left > 0) {
			  if (up_right > 0) {
				  if (low_right > 0) {
					  if (low_left > 0) {
						fout << "f " << up_left << "//" << up_left << " " << up_right 
							 << "//" << up_right << " " << low_left << "//" << low_left 
							 << "\n";
						fout << "f " << low_left << "//" << low_left << " " << up_right 
							 << "//" << up_right << " " << low_right << "//" << low_right 
							 << "\n";
					  }else {
						fout << "f " << up_left << "//" << up_left << " " << up_right
							 << "//" << up_right << " " << low_right << "//"
							 << low_right << "\n";
					  }
				  }else {
					  if (low_left > 0)
						  fout << "f " << up_left << "//" << up_left << " " << up_right 
							 << "//" << up_right << " " << low_left << "//" << low_left 
							 << "\n";
				  }
			  }else {
				  if (low_right > 0)
					  fout << "f " << up_left << "//" << up_left << " " << low_right
					  << "//" << low_right << " " << low_left << "//"
				  << low_left << "\n";
			  }
		  }else {
			  if ((low_left > 0)&&(up_right > 0))
				  fout << "f " << low_left << "//" << low_left << " " << up_right 
							 << "//" << up_right << " " << low_right << "//" << low_right 
							 << "\n";
		  }
		  
*/
		  if (((SU[i][j]._s >= s_lowcut)&&(SU[i][j]._s <= s_highcut))&&
			  ((SU[i][j]._t >= t_lowcut)&&(SU[i][j]._t <= t_highcut)))
		  {
			  up_left = 0;
			  temp ++;
		  }else up_left = i*S_SIZE + j + 1 - temp;

		  if (((SU[i][j+1]._s >= s_lowcut)&&(SU[i][j+1]._s <= s_highcut))&&
			  ((SU[i][j+1]._t >= t_lowcut)&&(SU[i][j+1]._t <= t_highcut)))
		  {
			  up_right = 0;
		  }else up_right = i*S_SIZE + j + 1 + 1 - temp;
		  if (((SU[i+1][j]._s >= s_lowcut)&&(SU[i+1][j]._s <= s_highcut))&&
			  ((SU[i+1][j]._t >= t_lowcut)&&(SU[i+1][j]._t <= t_highcut)))
		  {
			  low_left = 0;
		  }else low_left = i*S_SIZE + j + 1 + S_SIZE - temp;
		  if (((SU[i+1][j+1]._s >= s_lowcut)&&(SU[i+1][j+1]._s <= s_highcut))&&
			  ((SU[i+1][j+1]._t >= t_lowcut)&&(SU[i+1][j+1]._t <= t_highcut)))
		  {
			  low_right = 0;
		  }else low_right = i*S_SIZE + j + 1 + S_SIZE + 1 - temp;

		  if (up_left > 0) {
			  if (up_right > 0) {
				  if (low_right > 0) {
					  if (low_left > 0) {
						  if (((SU[i][j]._s >= s_lowcut)&&(SU[i][j]._s <= s_highcut)&&
							  (SU[i][j]._t < t_lowcut))||
							  ((SU[i+1][j]._s >= s_lowcut)&&(SU[i+1][j]._s <= s_highcut)&&
							  (SU[i+1][j]._t >= t_lowcut))) {
							fout << "f " << up_left << "//" << up_left << " " << up_right 
								 << "//" << up_right << " " << low_left - t_width 
								 << "//" << low_left - t_width << "\n";
							fout << "f " << low_left - t_width << "//" << low_left - t_width
								 << " " << up_right << "//" << up_right << " " 
								 << low_right - t_width << "//" << low_right - t_width << "\n";
						  }else {
							fout << "f " << up_left << "//" << up_left << " " << up_right 
								 << "//" << up_right << " " << low_left 
								 << "//" << low_left << "\n";
							fout << "f " << low_left << "//" << low_left 
								 << " " << up_right << "//" << up_right << " " 
								 << low_right << "//" << low_right << "\n";
						  }
					  }else {
						fout << "f " << up_left << "//" << up_left << " " << up_right
							 << "//" << up_right << " " << low_right - t_width << "//"
							 << low_right - t_width << "\n";
					  }
				  }else {
					  if (low_left > 0)
						  fout << "f " << up_left << "//" << up_left << " " << up_right 
							 << "//" << up_right << " " << low_left << "//" << low_left 
							 << "\n";
				  }
			  }else {
				  if (low_right > 0) 
					  fout << "f " << up_left << "//" << up_left << " " << low_right - t_width
					  << "//" << low_right - t_width << " " << low_left - t_width << "//"
					  << low_left - t_width << "\n";
			  }
		  }else {
			  if ((low_left > 0)&&(up_right > 0))
				  fout << "f " << low_left << "//" << low_left << " " << up_right 
							 << "//" << up_right << " " << low_right << "//" << low_right 
							 << "\n";
		  }
		  
		  
	  }
	}

	fout.close();
	cerr << ".now.\n";

}
	
void surface::give_st(double &s, double &t)
{
	s = SU[tpkey_i][tpkey_j]._s;
	t = SU[tpkey_i][tpkey_j]._t;
}
 
void surface::Gram_Schmidt(double rs[], double rt[], int &bad)
{
   double dot_prod, proj[3], u2_mag;
   double rs_mag = (double) sqrt(pow(rs[0],2) + pow(rs[1],2) + pow(rs[2],2));
   double rt_mag = (double) sqrt(pow(rt[0],2) + pow(rt[1],2) + pow(rt[2],2));
	
   if (rs_mag <= 0.0001) {
	   cerr << "Mag of t_plane vector rs is 0\n"; 
	   bad = 1;
   }

   rs[0] = (double) rs[0]/rs_mag;
   rs[1] = (double) rs[1]/rs_mag;
   rs[2] = (double) rs[2]/rs_mag;
//We now have u1, the first vector in the soon to be orthonormal basis.

   dot_prod = (double) rs[0]*rt[0] + rs[1]*rt[1] + rs[2]*rt[2];
   proj[0] = (double) dot_prod*rs[0];
   proj[1] = (double) dot_prod*rs[1];
   proj[2] = (double) dot_prod*rs[2];

   rt[0] -= proj[0];
   rt[1] -= proj[1];
   rt[2] -= proj[2];

   u2_mag = (double) sqrt(pow(rt[0],2) + pow(rt[1],2) + pow(rt[2],2));
//Now we have rt perp to rs (u1). Make it a unit vector.
   
   if (u2_mag <= 0.0001)  {
	   cerr << "Mag of t_plane vector u2 is 0\n";
	   bad = 2;
   }

   rt[0] = (double) rt[0]/u2_mag;
   rt[1] = (double) rt[1]/u2_mag;
   rt[2] = (double) rt[2]/u2_mag;

//And finally, rs and rt are orthonormal...whew!!!
 
}  
    
  
void surface::extremes(double &x, double &y, double &u, double &v) 
//Sets the parameters to the extremes elements of the instanciation.
{
        x = min_x;
        y = min_y;
        u = max_x;
        v = max_y;
}

void surface::eye_extremes_util(transformation *Current)
//This is a hw5 specific utility that puts in min_x, etc, the extreme
//values of x and y once they've been transformed by Current.  This had 
//to be a member function, as the implementation of the edge_list is 
//private.
{
   edge *curpent_edge = surf_front;
   double temp[4];
   double x, y, u, v;

   while (curpent_edge != NULL) {
        copy_vert_s(curpent_edge->v1s, curpent_edge->v1t, temp);
	Current->Apply(temp, 1);
        
        x = temp[0];
        y = temp[1];

	copy_vert_s(curpent_edge->v2s, curpent_edge->v2t, temp);
	Current->Apply(temp, 1);

        u = temp[0];
        v = temp[1];

	test_for_extremes (x, y, u, v);

        curpent_edge = curpent_edge->next;
   }
}			

void surface::test_for_extremes(double &x, double &y, double &u, double &v) 
//Sets the extremes elements of the instanciation is required.
{
   if (x < min_x) {
        min_x = x;
   }else if (x > max_x) {
        max_x = x;
   }

   if (u < min_x) {
        min_x = u;
   }else if (u > max_x) {
        max_x = u;
   }

   if (y < min_y) {
        min_y = y;
   }else if (y > max_y) {
        max_y = y;
   }

   if (v < min_y) {
        min_y = v;
   }else if (v > max_y) {
        max_y = v;
   }
}		  
			
void surface::deg0_undef(double var[])
{
	if(var[3] == 0) var[3] = 1;
	if(var[5] == 0) var[5] = 1;
	if(var[7] == 0) var[7] = 1;
	if(var[9] == 0) var[9] = 1;
}

void surface::find_normal(int i, int j, double n[])
{
	double rs[3], rt[3], tempx, tempy, tempz, mag, temp1, temp2, temp3;
	double s0 = SU[i][j]._s, t0 = SU[i][j]._t;
	int bad = -1;


//Here is a tough part.  I've hardcoded the proper partial derivatives,
//so depending on the deg of the x y z, this is ugly but efficient.

   if (deg == 3) {
	rs[0] = (double) 3*x[0]*pow(s0,2) + 2*x[1]*s0*t0
			+ x[2]*pow(t0,2) + 2*x[4]*s0 + x[5]*t0 + x[7];
	rs[1] = (double) 3*y[0]*pow(s0,2) + 2*y[1]*s0*t0
                        + y[2]*pow(t0,2) + 2*y[4]*s0 + y[5]*t0 + y[7];
	rs[2] = (double) 3*z[0]*pow(s0,2) + 2*z[1]*s0*t0
                        + z[2]*pow(t0,2) + 2*z[4]*s0 + z[5]*t0 + z[7];
	rt[0] = (double) x[1]*pow(s0,2) + 2*x[2]*s0*t0
			+ 3*x[3]*pow(t0,2) + x[5]*s0 + 2*x[6]*t0 + x[8];
	rt[1] = (double) y[1]*pow(s0,2) + 2*y[2]*s0*t0
                        + 3*y[3]*pow(t0,2) + y[5]*s0 + 2*y[6]*t0 + y[8];
	rt[2] = (double) z[1]*pow(s0,2) + 2*z[2]*s0*t0
                        + 3*z[3]*pow(t0,2) + z[5]*s0 + 2*z[6]*t0 + z[8];
   }else if (deg == 2) {
	rs[0] = (double) 2*x[0]*s0 + x[1]*t0 + x[3];
	rs[1] = (double) 2*y[0]*s0 + y[1]*t0 + y[3];
	rs[2] = (double) 2*z[0]*s0 + z[1]*t0 + z[3];
	rt[0] = (double) x[1]*s0 + 2*x[2]*t0 + x[4];
	rt[1] = (double) y[1]*s0 + 2*y[2]*t0 + y[4];
	rt[2] = (double) z[1]*s0 + 2*z[2]*t0 + z[4];
   }else if (deg == 0) {
	temp1 = (pow(s0,2) + pow(t0,2))/x[3];
	temp1 = (-2.0*x[2]/x[3])*sin(temp1);
	temp2 = s0*t0/x[5];
	temp2 = (-1.0*x[4]/x[5])*sin(temp2);
	temp3 = s0/x[7];
	temp3 = (-1.0*x[6]/x[7])*sin(temp3);
	rs[0] = (double) x[0] + s0*temp1 + t0*temp2 + temp3;
	temp3 = t0/x[9];
	temp3 = (-1.0*x[8]/x[9])*sin(temp3);
	rt[0] = (double) x[1] + t0*temp1 + s0*temp2 + temp3;

	temp1 = (pow(s0,2) + pow(t0,2))/y[3];
	temp1 = (-2.0*y[2]/y[3])*sin(temp1);
	temp2 = s0*t0/y[5];
	temp2 = (-1.0*y[4]/y[5])*sin(temp2);
	temp3 = s0/y[7];
	temp3 = (-1.0*y[6]/y[7])*sin(temp3);
	rs[1] = (double) y[0] + s0*temp1 + t0*temp2 + temp3;
	temp3 = t0/y[9];
	temp3 = (-1.0*y[8]/y[9])*sin(temp3);
	rt[1] = (double) y[1] + t0*temp1 + s0*temp2 + temp3;

	temp1 = (pow(s0,2) + pow(t0,2))/z[3];
	temp1 = (-2.0*z[2]/z[3])*sin(temp1);
	temp2 = s0*t0/z[5];
	temp2 = (-1.0*z[4]/z[5])*sin(temp2);
	temp3 = s0/z[7];
	temp3 = (-1.0*z[6]/z[7])*sin(temp3);
	rs[2] = (double) z[0] + s0*temp1 + t0*temp2 + temp3;
	temp3 = t0/z[9];
	temp3 = (-1.0*z[8]/z[9])*sin(temp3);
	rt[2] = (double) z[1] + t0*temp1 + s0*temp2 + temp3;
   }

   Gram_Schmidt(rs, rt, bad);
//temp1*temp2*
   tempx = (rs[1]*rt[2] - rt[1]*rs[2]);
   tempy = -1.0*(rs[0]*rt[2] - rt[0]*rs[2]);
   tempz = (rs[0]*rt[1] - rt[0]*rs[1]);

   mag = pow(tempx, 2) + pow(tempy, 2) + pow(tempz, 2);
   mag = sqrt(mag);

   if ((bad == 1)||(bad == 2)) {
		n[0] = 0;
	    n[1] = 0;
	    n[2] = 0;
   }else {

   n[0] = tempx/mag;
   n[1] = tempy/mag;
   n[2] = tempz/mag;
   }
}